"""OZI Standard API project creation script.
Required-only:
    project.author

Validated:
    project.name
    project.email
    project.summary
    project.license_spdx
    project.license
    project.topic
    project.status

"""
import argparse
from datetime import datetime, timezone
from pathlib import Path
import re
from typing import NoReturn, Union
from urllib.parse import urlparse

from jinja2 import Environment, FileSystemLoader, select_autoescape
from spdx_license_list import LICENSES
from email_validator import validate_email, EmailNotValidError
from pyparsing import Regex, ParseException

from .assets import status, topic

root_templates = [
    '.gitignore',
    'meson.build',
    'meson.options',
    'PKG-INFO',
    'pyproject.toml',
    'README.rst',
]
source_templates = [
    'project.name/__init__.py',
    'project.name/__init__.pyi',
    'project.name/meson.build',
]
env = Environment(
    loader=FileSystemLoader(['templates', 'templates/project.name']),
    autoescape=select_autoescape(),
    enable_async=True,
)
parser = argparse.ArgumentParser(description='OZI Python Project Packaging Quick-start')
parser.add_argument('--name', type=str, help='name of project', required=True)
parser.add_argument('--author', type=str, help='author of project', required=True)
parser.add_argument('--email', type=str, help='valid author email', required=True)
parser.add_argument('--summary', type=str, help='short summary', required=True)
parser.add_argument('--homepage', type=str, help='homepage URL', required=True)
parser.add_argument(
    'target', type=str, help='target directory for new project',
)
parser.add_argument(
    '--license-spdx',
    type=str,
    choices=[k for k, v in LICENSES.items() if v.deprecated_id is False],
    metavar='ID',
    help='SPDX short ID',
)
parser.add_argument(
    '--license', type=str, help='license classifier (Pending PEP 639 Deprecation)'
)
parser.add_argument(
    '--copyright-head',
    type=str,
    default='',
    help='copyright header string',
    metavar='COPYRIGHT_HEAD="Copyright {year}, {project.author}\\nSee LICENSE..."',
)
parser.add_argument(
    '--topic',
    choices=topic.choices,
    default='Utilities',
    help='Python package topic',
    metavar='TOPIC="Utilities"',
    type=str,
)
parser.add_argument(
    '--status',
    choices=status.choices,
    default='1 - Planning',
    help='Python package status',
    metavar='STATUS="1 - Planning"',
    type=str,
)


def main() -> Union[NoReturn, None]:
    """Main ozi.new entrypoint."""
    project = parser.parse_args()

    if len(project.copyright_head) == 0:
        copyright_year = datetime.now(tz=datetime.now(timezone.utc).astimezone().tzinfo).year
        project.copyright_head = '\n'.join(
            [
                f'Copyright {copyright_year}, {project.author}',
                'See LICENSE.txt in the project root for details.',
            ]
        )

    if len(project.summary) > 512:
        raise ValueError('Project summary exceeds 512 characters in length.')

    try:
        emailinfo = validate_email(project.email, check_deliverability=True)
    except EmailNotValidError as e:
        raise ValueError(str(e), 'Invalid maintainer email format or domain unreachable.')

    try:
        Regex('^([A-Z0-9]|[A-Z0-9][A-Z0-9._-]*[A-Z0-9])$', re.IGNORECASE).parse_string(
            project.name
        )
    except ParseException as e:
        raise ValueError(str(e), 'Invalid project name.')

    home_url = urlparse(project.homepage).geturl
    if home_url.scheme != 'https':
        raise ValueError('Homepage url scheme unsupported.')

    if home_url.netloc == '':
        raise ValueError('Homepage url netloc cound not be parsed.')

    project.name = re.sub(r'[-_.]+', '-', project.name).lower()
    project.email = emailinfo.normalized
    project.target = Path(project.target)

    env.globals = {'project': vars(project)}
    Path(project.target, project.name).mkdir()
    Path(project.target, '.github', 'workflows').mkdir(parents=True)
    Path(project.target, 'subprojects').mkdir()
    Path(project.target, 'tests').mkdir()

    for filename in root_templates:
        template = env.get_template(f'{filename}.j2')
        with open(project.target / filename, 'w') as f:
            f.write(template.render())

    for filename in source_templates:
        template = env.get_template(f'{filename}.j2')
        with open(project.target / filename, 'w') as f:
            f.write(template.render())


if __name__ == '__main__':
    main()



# noqa: INP001
# Copyright 2023 Ross J. Duff MSc
# The copyright holder licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at

#   http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""Check if the README.rst file is synchronized with PKG-INFO."""
import difflib
import email
import os
import pathlib
import sys

#
try:
    import toml  # noqa: F401
except ImportError:
    import tomli as toml  # noqa: F401
# pylint: disable=consider-using-with
source = '/' / pathlib.Path(
    os.path.relpath(
        os.path.join('/', os.environ.get('MESON_SOURCE_ROOT', os.path.relpath('..'))), '/'
    )
)
pkg_info_file = open(source / 'PKG-INFO', 'r', encoding='utf-8')
pkg_info = email.message_from_file(pkg_info_file).get_payload()
pkg_info_file.close()
readme_file = open(source / 'README.rst', 'r', encoding='utf-8')
readme = readme_file.read()
readme_file.close()
project_file = open(source / 'pyproject.toml', 'r', encoding='utf-8')
pyproject_toml = toml.loads(project_file.read())
project_file.close()
setuptools_scm = pyproject_toml.get('tool', {}).get('setuptools_scm', {})
write_to_template = email.message_from_string(
    setuptools_scm.get('write_to_template', '')
).get_payload()
DIFF1 = str().join(
    difflib.context_diff(pkg_info, readme, tofile='PKG-INFO', fromfile='README.rst')
)
DIFF2 = str().join(
    difflib.context_diff(
        readme, write_to_template, tofile='pyproject.toml', fromfile='README.rst'
    )
)
DIFF3 = str().join(
    difflib.context_diff(
        write_to_template, pkg_info, tofile='pyproject.toml', fromfile='PKG-INFO'
    )
)
if DIFF1 != '':
    print(DIFF1)
    sys.exit(len(DIFF1))
elif DIFF2 != '':
    print(DIFF2)
    sys.exit(len(DIFF2))
elif DIFF3 != '':
    print(DIFF3)
    sys.exit(len(DIFF3))
